# -*- coding: utf-8 -*-
"""
Created on Thu Oct 28 04:33:22 2023

@author: Changpuak
"""
import serial
import time
import matplotlib.pyplot as plt
import numpy as np

# PLEASE DON'T FORGET TO CHANGE THE NUMBER OF THE COM PORT
Tanachai = serial.Serial(port='COM124', baudrate=115200, timeout=.9)

time.sleep(5)               # Arduinos reset, wait to start up

Drain_Start = 0.0
Drain_Stop = + 2.0          # Allow some headroom for Shunt Resistor Drop
Drain_Step = 0.1

Source_Volts = 0.0

WAIT = 0.5                  # seconds

I = []  # Current
U = []  # Voltage

Drain = Drain_Start

def CHAT_TANACHAIMOD(x):
    Tanachai.write(x.encode('utf-8'))
    time.sleep(WAIT)
    data = []
    line = (Tanachai.readline())
    while len(line) > 0:
        data.append(line)
        line = Tanachai.readline()
        line = line.decode('utf-8')
    return data

def EMPTY_TANACHAIMOD():
    time.sleep(WAIT)
    x = b'avanti'
    while x != b'':
        x = Tanachai.readline()

try:
    print("STARTING SWEEP ...")
    CHAT_TANACHAIMOD('*IDN?\n')
    EMPTY_TANACHAIMOD()
    while Drain <= Drain_Stop:
        print(str(round(Drain, 3)))
        # SET CURRENT
        CHAT_TANACHAIMOD('SETDRAIN:'+str((round(Drain, 2)))+'\n')
        # GET SOURCE VOLTAGE
        Level = str(CHAT_TANACHAIMOD('SOURCE?\n'))
        Source_Volts = 1000 * float(Level[3:-9])
        print(str(round(Source_Volts, 3)))
        U.append(Drain - (Source_Volts/1000))       # FILL ARRAY
        I.append(Source_Volts)
        Drain += Drain_Step
        plt.style.use('bmh')
        # plot - yes it is updated in the loop
        fig, ax = plt.subplots(1,1)    
        ax.plot(U, I, linewidth=1.0, color='#0000FF')
        ax.set_title('SB 340 Schottky Diode')
        ax.set_ylabel('Current [mA]')
        ax.set_xlabel('Voltage [V]')
        ax.set(xlim=(Drain_Start, 1.0), 
                xticks=np.arange(Drain_Start, 1.0, 0.1),
                ylim=(-10, +1001), yticks=np.arange(200, 1201, 200))
        plt.show()

finally:
    CHAT_TANACHAIMOD('SETDRAIN:0.0\n')
    Tanachai.close()
    print("CONNECTION CLOSED.")


